// Copyright 2004 Edtech Production
// http://dev.manga-france.com
// Version 1.2 du 09 Mars 2005

unit EdtTranslator;

interface

uses
  Windows, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ActnList, XPStyleActnCtrls, ActnMan, ToolWin,
  ActnCtrls, ActnMenus, ComCtrls, Menus, StrUtils, ExtCtrls, EdtURLLabel,
  ActnPopup;

type
  TTransType = (ttComplete, ttPartial);
  TEdtTranslator = class(TComponent)
  private
    Translation: array of array of string;
    fLanguage: string;
    fPath: string;
    fOnTranslate: TNotifyEvent;
    fOnBeforeTranslate: TNotifyEvent;
    fOnAfterTranslate: TNotifyEvent;
    fReady: boolean;
    fDebugMode: boolean;
    fTransType: TTransType;
    procedure SetLanguage(const Value: string);
    function LireFichier(Fichier: string): boolean;
    function TraduireBarreBouton(Menu: TActionToolBar): boolean;
    function TraduireBarreMenu(Menu: TActionMainMenuBar): boolean;
    function TraduireComposants(Composant: TComponent): boolean;
    function TraduireItems(Item: TMenuItem): boolean;
    function TraduireListe(Liste: TListView): boolean;
    function TraduireMenuItems(Item: TActionClientItem): boolean;
    function TraduireMenuPopup(Menu: TPopupMenu): boolean;
    function TraduireStatusBar(Barre: TStatusBar): boolean;
    procedure SetPath(const Value: string);
    function TraduireMainMenu(Menu: TMainMenu): boolean;
    procedure SetDebugMode(const Value: boolean);
    procedure SetTransType(const Value: TTransType);
    { Dclarations prives }
  protected
    { Dclarations protges }
  public
    { Dclarations publiques }
    Property Ready: boolean read fReady default False;
    procedure Translate();
    function ReadData(Line: integer; NumString: integer): string;
    constructor Create(AOwner:TComponent); override;
  published
    { Dclarations publies }
    property LanguageFile: string read fLanguage write SetLanguage;
    property PathFiles: string read fPath write SetPath;
    property OnTranslate: TNotifyEvent read fOnTranslate write FOnTranslate;
    property OnBeforeTranslate: TNotifyEvent read fOnBeforeTranslate write FOnBeforeTranslate;
    property OnAfterTranslate: TNotifyEvent read fOnAfterTranslate write FOnAfterTranslate;
    property DebugMode: boolean read fDebugMode write SetDebugMode default False;
    property TranlationType: TTransType read fTransType write SetTransType default ttComplete;
  end;

procedure Register;

implementation

{ TEdtTranslator }

procedure Register;
begin
  RegisterComponents('MoneyLine', [TEdtTranslator]);
end;

// Translation des TActionItem
function TEdtTranslator.TraduireMenuItems(Item: TActionClientItem): boolean;
var i: integer;
begin
for i:=0 to Item.Items.Count - 1 do
  begin
  if Item.Items[i].Tag <> 0 then
    begin
    Item.Items[i].Caption := ReadData(Item.Items[i].Tag, 0);
    if Item.Items[i].Items.Count > 0 then TraduireMenuItems(Item.Items[i]);
    end;
  end;
Result := True;
end;

// Translation des TActionMainMenuBar
function TEdtTranslator.TraduireBarreMenu(Menu: TActionMainMenuBar): boolean;
var i: integer;
begin
for i:=0 to Menu.ActionClient.Items.Count - 1 do
  begin
  if Menu.ActionClient.Items[i].Tag <> 0 then
    begin
    Menu.ActionClient.Items[i].Caption := ReadData(Menu.ActionClient.Items[i].Tag, 0);
    if Menu.ActionClient.Items[i].Items.Count > 0 then TraduireMenuItems(Menu.ActionClient.Items[i]);
    end;
  end;
Result := True;
end;

// Translation des TActionToolBar
function TEdtTranslator.TraduireBarreBouton(Menu: TActionToolBar): boolean;
var i: integer;
begin
for i:=0 to Menu.ActionClient.Items.Count - 1 do
  begin
  if Menu.ActionClient.Items[i].Tag <> 0 then
    begin
    Menu.ActionClient.Items[i].Caption := ReadData(Menu.ActionClient.Items[i].Tag, 0);
    if Menu.ActionClient.Items[i].Items.Count > 0 then TraduireMenuItems(Menu.ActionClient.Items[i]);
    end;
  end;
Result := True;
end;

// Translation des TMenuItem
function TEdtTranslator.TraduireItems(Item: TMenuItem): boolean;
var i: integer;
begin
for i:=0 to Item.Count - 1 do
  begin
  if Item.Items[i].Tag <> 0 then
    begin
    Item.Items[i].Caption := ReadData(Item.Items[i].Tag, 0);
    if Item.Items[i].Count > 0 then TraduireItems(Item.Items[i]);
    end;
  end;
Result := True;
end;

// Translation des MainMenu
function TEdtTranslator.TraduireMainMenu(Menu: TMainMenu): boolean;
var i: integer;
begin
for i:=0 to Menu.Items.Count - 1 do
  begin
  if Menu.Items[i].Tag <> 0 then
    begin
    Menu.Items[i].Caption := ReadData(Menu.Items[i].Tag, 0);
    if Menu.Items[i].Count > 0 then TraduireItems(Menu.Items[i]);
    end;
  end;
Result := True;
end;

// Translation des menus Popup
function TEdtTranslator.TraduireMenuPopup(Menu: TPopupMenu): boolean;
var i: integer;
begin
for i:=0 to Menu.Items.Count - 1 do
  begin
  if Menu.Items[i].Tag <> 0 then
    begin
    Menu.Items[i].Caption := ReadData(Menu.Items[i].Tag, 0);
    if Menu.Items[i].Count > 0 then TraduireItems(Menu.Items[i]);
    end;
  end;
Result := True;
end;

// Translation des TListView
function TEdtTranslator.TraduireListe(Liste: TListView): boolean;
var i: integer;
begin
for i:=0 to Liste.Columns.Count - 1 do if Liste.Columns.Items[i].Tag <> 0 then
Liste.Columns.Items[i].Caption := ReadData(Liste.Columns.Items[i].Tag, 0);
Result := True;
end;

// Traduction des TStatusBar
function TEdtTranslator.TraduireStatusBar(Barre: TStatusBar): boolean;
var i: integer;
begin
Barre.Hint := ReadData(Barre.Tag, 0);
for i:=0 to Barre.Panels.Count - 1 do if ReadData(Barre.Tag, i + 1) <> '' then Barre.Panels.Items[i].Text := ReadData(Barre.Tag, i + 1);
Result := True;
end;

// Traduction des composants classiques
function TEdtTranslator.TraduireComposants(Composant: TComponent): boolean;
var i, NbItems: integer;
begin
if Composant.Tag <> 0 then
  begin
  if Composant.ClassType = TButton then
    begin
    (Composant as TButton).Caption := ReadData(Composant.Tag, 0);
    (Composant as TButton).Hint := ReadData(Composant.Tag, 1);
    end;
  if Composant.ClassType = TPanel then
    begin
    (Composant as TPanel).Caption := ReadData(Composant.Tag, 0);
    (Composant as TPanel).Hint := ReadData(Composant.Tag, 1);
    end;
  if Composant.ClassType = TGroupBox then
    begin
    (Composant as TGroupBox).Caption := ReadData(Composant.Tag, 0);
    (Composant as TGroupBox).Hint := ReadData(Composant.Tag, 1);
    end;
  if Composant.ClassType = TRadioGroup then
    begin
    (Composant as TRadioGroup).Caption := ReadData(Composant.Tag, 0);
    (Composant as TRadioGroup).Hint := ReadData(Composant.Tag, 1);
    end;
  if Composant.ClassType = TTabSheet then
    begin
    (Composant as TTabSheet).Caption := ReadData(Composant.Tag, 0);
    (Composant as TTabSheet).Hint := ReadData(Composant.Tag, 1);
    end;
  if Composant.ClassType = TCheckBox then
    begin
    (Composant as TCheckBox).Caption := ReadData(Composant.Tag, 0);
    (Composant as TCheckBox).Hint := ReadData(Composant.Tag, 1);
    end;
  if Composant.ClassType = TRadioButton then
    begin
    (Composant as TRadioButton).Caption := ReadData(Composant.Tag, 0);
    (Composant as TRadioButton).Hint := ReadData(Composant.Tag, 1);
    end;
  if Composant.ClassType = TLabel then
    begin
    (Composant as TLabel).Caption := ReadData(Composant.Tag, 0);
    (Composant as TLabel).Hint := ReadData(Composant.Tag, 1);
    end;
  if Composant.ClassType = TEdtURLLabel then
    begin
    (Composant as TEdtURLLabel).Caption := ReadData(Composant.Tag, 0);
    (Composant as TEdtURLLabel).Hint := ReadData(Composant.Tag, 1);
    end;
  if Composant.ClassType = TAction then
    begin
    (Composant as TAction).Caption := ReadData(Composant.Tag, 0);
    (Composant as TAction).Hint := ReadData(Composant.Tag, 1);
    end;
  if Composant.ClassType = TEdit then
    begin
    (Composant as TEdit).Text := ReadData(Composant.Tag, 0);
    (Composant as TEdit).Hint := ReadData(Composant.Tag, 1);
    end;
  if Composant.ClassType = TOpenDialog then
    (Composant as TOpenDialog).Title:= ReadData(Composant.Tag, 0);
  if Composant.ClassType = TSaveDialog then
    (Composant as TSaveDialog).Title:= ReadData(Composant.Tag, 0);
  if Composant.ClassType = TCombobox then
    begin
    NbItems := (Composant as TCombobox).Items.Count;
    (Composant as TCombobox).Clear;
    for i := 0 to NbItems - 1 do (Composant as TCombobox).AddItem(ReadData(Composant.Tag, i), Self);
    if (Composant as TCombobox).Items.Count > 0 then (Composant as TCombobox).ItemIndex := 0;
    end;
  if Composant.ClassType = TListbox then
    begin
    NbItems := (Composant as TListbox).Items.Count;
    (Composant as TListbox).Clear;
    for i := 0 to NbItems - 1 do (Composant as TListbox).AddItem(ReadData(Composant.Tag, i), Self);
    if (Composant as TListbox).Items.Count > 0 then (Composant as TListbox).ItemIndex := 0;
    end;
  if Composant.ClassType = TTabControl then
    begin
    NbItems := (Composant as TTabControl).Tabs.Count;
    (Composant as TTabControl).Tabs.Clear;
    for i := 0 to NbItems - 1do (Composant as TTabControl).Tabs.Add(ReadData(Composant.Tag, i));
    if (Composant as TTabControl).Tabs.Count > 0 then (Composant as TTabControl).TabIndex := 0;
    end;
  end;
if Composant.ClassType = TActionMainMenuBar then TraduireBarreMenu((Composant as TActionMainMenuBar));
if Composant.ClassType = TActionToolBar then TraduireBarreBouton((Composant as TActionToolBar));
if Composant.ClassType = TMainMenu then TraduireMainMenu((Composant as TMainMenu));
if Composant.ClassType = TPopupMenu then TraduireMenuPopup((Composant as TPopupMenu));
if Composant.ClassType = TPopupActionBar then TraduireMenuPopup((Composant as TPopupMenu));
if Composant.ClassType = TStatusBar then TraduireStatusBar((Composant as TStatusBar));
if Composant.ClassType = TListView then TraduireListe((Composant as TListView));
if Composant.ComponentCount > 0 then
  begin
  for i:=0 to Composant.ComponentCount -1 do TraduireComposants(Composant.Components[i]);
  end;
Result := True;
end;

function TEdtTranslator.LireFichier(Fichier: string): boolean;
var FichierLng: TextFile;
    Chaine: string;
    Numero, SousNumero, Position, i: integer;
begin
  AssignFile(FichierLng, Fichier);
  try
    Reset(FichierLng);
  except on EInOutError do
    begin
      Result := False;
      Exit;
    end;
  end;
  fReady := False;
  Translation := nil;
  SetLength(Translation, 1,1);
  while not eof(FichierLng) do
  begin
    Readln(FichierLng, Chaine);
    Chaine := Trim(Chaine);
    if (Length(Chaine) > 0) and (Chaine[1] = '#') then
    begin

      Numero := StrToInt(Copy(Chaine, 2, 4));
      SousNumero := StrToInt(Copy(Chaine, 7, 4));
      if High(Translation) < Numero then SetLength(Translation, Numero + 1);
      if High(Translation[Numero]) < SousNumero then SetLength(Translation[Numero], SousNumero + 1);
      Translation[Numero, SousNumero] := Trim(Copy(Chaine, 13, Length(Chaine) - 12));
    end;
  end;
  CloseFile(FichierLng);
  fReady := True;
  Result := True;
end;

// Lecture d'une donne du tableau de traductions
function TEdtTranslator.ReadData(Line, NumString: integer): string;
begin
if Assigned(Translation[Line]) and (Line < Length(Translation)) and (Length(Translation[Line]) > 0) and (High(Translation[Line]) >= NumString) then
  Result := Translation[Line, NumString]
else
  begin
  if (fDebugMode) and (NumString <> 1) then MessageBox(Application.Handle, Pchar('Translation ' + IntToStr(Line) + ', ' + IntToStr(NumString) + ' not found.'), 'Error in language file', MB_ICONEXCLAMATION);
  Result := '';
  end;
end;

procedure TEdtTranslator.SetLanguage(const Value: string);
begin
fLanguage := Value;
end;

procedure TEdtTranslator.Translate();
var Pere: TClass;
    Composant: TComponent;
    i: integer;
    Chemin: string;
    FicheMere: TForm;
begin
if not (csDesigning in ComponentState) and (fLanguage <> '') then
  begin
  if assigned(fOnBeforeTranslate) then OnBeforeTranslate(Self);
  if Length(fPath) > 0 then
    if fPath[Length(fPath)] <> '\' then Chemin := fPath + '\' + fLanguage else Chemin := fPath + fLanguage
  else Chemin := fLanguage;
  if LireFichier(Chemin) then
    begin
    if Assigned(fOnTranslate) then OnTranslate(Self);
    if fTransType = ttComplete then
      begin
      for i:=1 to Application.ComponentCount - 1 do
        begin
        Composant := Application.Components[i];
        Pere := Composant.ClassType;
        if (Composant.Tag <> 0) and (Composant.ClassType = Pere) then (Composant as TForm).Caption := ReadData(Composant.Tag, 0);
        TraduireComposants(Composant);
        end;
      end
    else
      begin
      FicheMere := (Owner as TForm);
      for i:=0 to FicheMere.ComponentCount - 1 do
        begin
        Composant := FicheMere.Components[i];
        TraduireComposants(Composant);
        end;
      end;
    if Assigned(fOnAfterTranslate) then OnAfterTranslate(Self);
    end;
  end;
end;

procedure TEdtTranslator.SetPath(const Value: string);
begin
  fPath := Value;
end;

procedure TEdtTranslator.SetDebugMode(const Value: boolean);
begin
  fDebugMode := Value;
end;

constructor TEdtTranslator.Create(AOwner: TComponent);
begin
inherited;
fReady := False;
fDebugMode := False;
fTransType := ttComplete;
end;

procedure TEdtTranslator.SetTransType(const Value: TTransType);
begin
  fTransType := Value;
end;

end.
